import ast
import json

from ._base import InspectorBase
from tornado.web import RequestHandler


class PythonInspector(InspectorBase):
    def __init__(self):
        super().__init__()

    def available(self):
        return True

    def get_variables(self, notebook, cell_idx: int):
        cell_code = notebook['content']['cells'][cell_idx]['source']

        variables = set()

        root = ast.parse(cell_code)
        for node in ast.walk(root):
            #  Only variables that are loaded should be changed
            if isinstance(node, ast.Name) and isinstance(node.ctx, ast.Load):
                variables.add(node.id)

        return list(variables)

    def run_pre_cell(self, config, req: RequestHandler) -> str:
        variables = config['variables']
        
        if req.request.method == 'POST':
            json_body = json.loads(req.request.body)
        else:
            json_body = {}

        code = ''

        for v in variables.keys():
            var_type = variables[v]

            if var_type == 'query':
                arg_string = req.get_argument(v, None)
                if arg_string is not None:
                    code += f'{v} = {arg_string}\n'
            if var_type == 'post' and req.request.method == 'POST':
                arg_string = json_body.get(v, None)

                if arg_string is not None:
                    code += f'{v} = {arg_string}\n'

        return code


inspector = PythonInspector



import json

from typing import List
from queue import Empty

from jupyter_client import manager as kernel_manager, kernelspec

import logging

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

# create console handler and set level to debug
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# add formatter to ch
ch.setFormatter(formatter)

# add ch to logger
logger.addHandler(ch)

class NotebookRunner:
    def __init__(self, notebook_path):
        self.path = notebook_path

        with open(notebook_path, 'r') as notebook_file:
            self.json = json.load(notebook_file)

            self.kernel_name = self.json['metadata']['kernelspec']['name']
            self.cells = self.json['cells']

        # Might raise kernelspec.NoSuchKernel
        m, c = kernel_manager.start_new_kernel(kernel_name=self.kernel_name)

        self.manager = m
        self.client = c

        self.run_before = {}

    def set_run_before(self, cell_idx, source):
        self.run_before[cell_idx] = source

    def shutdown(self):
        self.client.shutdown()

    def execute_code_and_wait(self, command):
        logger.info('execute_code_and_wait, command: '+command)
        cmd_id = self.client.execute(command)

        state = 'busy'
        ret_content = {}

        #  TODO this only returns the last message of one type. I.e. when a
        #  a cell draws multiple images only the last one will be sent.

        while state != 'idle':
            try:
                msg = self.client.get_iopub_msg(timeout=10)

                if msg['parent_header']['msg_id'] != cmd_id:
                    continue

                if not 'content' in msg: 
                    continue

                msg_type = msg['msg_type']

                if msg_type == 'status':
                    state = msg['content']['execution_state']

                if state == 'idle':
                    break

                content = msg['content']
                if msg_type == "stream":
                    ret_content[content['name']] = content['text']
                elif msg_type == 'execute_result':
                    ret_content['result'] = content
                elif msg_type == 'display_data':
                    ret_content['display'] = content
                elif msg_type == 'error':
                    ret_content['error'] = content
            except Empty:
                logger.info('except Empty:')
                return {}

        return ret_content

    def run(self):
        self.run_until()

    def run_until(self, until_cell_idx=None) -> List[any]:
        results = []
        logger.info('run_until')
        for idx, cell in enumerate(self.cells):
            if cell['cell_type'] == 'code':
                if idx in self.run_before:
                    self.execute_code_and_wait(self.run_before[idx])

                res = self.execute_code_and_wait("".join(cell['source']))
                results.append(res)
            else:
                results.append({})

            if idx == until_cell_idx:
                break

        return results



import json
import os
import tempfile
import shutil
import importlib

from typing import Optional

import docker
from notebook.base.handlers import IPythonHandler, APIHandler, HTTPError
from notebook.utils import url_path_join
from pigar.core import parse_packages

from ..container.docker_service import DockerService
from .base_handler import BaseHandler


BASE_STRING = """# Autogenerated by FAIR-Cells, please adjust.
channels:
  - conda-forge
"""

def determine_packages(dir=os.getcwd()):
    libs, guess = parse_packages(dir)
    determined = [(key, libs[key].version) for key in libs.keys()]
    determined.extend([(key, None) for key in guess.keys()])

    return determined


def write_requirements(packages):
    if len(packages) == 0:
        return ""

    requirements = "dependencies:\n"
    requirements += "  - pip:\n"

    for lib, version in packages:
        if version:
            requirements += f"    - {lib}=={version}\n"
        else:
            requirements += f"    - {lib}\n"

    return requirements



class EnvironmentHandler(BaseHandler):
    def get(self, path):        
        # notebook = self.contents_manager.get(path, content=True)
        notebook_path = os.path.join(os.getcwd(), path)

        #  Create a temporary dir which will be our build context.
        with tempfile.TemporaryDirectory() as tmpdir:
            shutil.copyfile(notebook_path, tmpdir + "/notebook.ipynb")

            packages = determine_packages(tmpdir)
        
        reqs = write_requirements(packages)

        self.finish(json.dumps({
            'data': BASE_STRING + reqs
        }))



import json
import logging
import requests
import sys

from .base_handler import BaseHandler
from ..container.docker_service import DockerService

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)



class LoginDockerRepositoryHandler(BaseHandler):

    def post(self, path):
        body = self.get_json_body()

        docker_repository = body.get('dockerRepository')
        docker_username = body.get('dockerUsername')
        docker_token = body.get('dockerToken')
        logging.info("docker_repository: " + str(docker_repository))
        logging.info("docker_username: " + str(docker_username))
        logging.info("docker_token: " + str(docker_token))

        ds = DockerService()
        # try:
        logging.info('username: ' + docker_username + 'password: ' + docker_token + 'registry:' + docker_repository)
        resp = ds.login(url=docker_repository,username=docker_username,token=docker_token)
        logging.info("resp: " + str(resp))
        self.finish(json.dumps(resp))
        # except requests.exceptions.HTTPError as e:
        #     logging.warning("e: " + str(e))
        #     error = self.write_error(e.status_code, exc_info=sys.exc_info())
        #     logging.warning("error: " + str(error))




import json
import os

from typing import Optional

import docker
from notebook.base.handlers import APIHandler, HTTPError

from ..container.docker_service import DockerService
from .base_handler import BaseHandler


class CommandHandler(BaseHandler):
    def post(self, image_name, command):
        if command == 'run':
            return self._run(image_name)
        elif command == 'stop':
            return self._stop(image_name)

        raise HTTPError(400, 'no_cmd')

    def get(self, image_name, command):
        if command == 'status':
            return self._status(image_name)

        raise HTTPError(400, 'no_cmd')

    def _stop(self, image_name):
        client = docker.from_env()
        
        try:
            cc = DockerService()
            status = cc.stop_image(image=image_name)
            if not status:
                status = ['not_found']
        except docker.errors.NotFound:
            status = 'not_found'
        finally:
            self.finish(json.dumps({
                'data': status[0]
            }))

    def _run(self, image_name):
        body = self.get_json_body()
        port = self._int_body('port', 10000)

        if port is None:
            raise HTTPError(400, 'def')


        cc = DockerService()
        container = cc.run_container(port=port,image=image_name)

        self.finish(json.dumps({
            'data': container.status
        }))

    def _status(self, image_name):
        client = docker.from_env()

        try:
            cc = DockerService()
            status = cc.get_image_status(image_name)
            if not status:
                status=['not_found']
        except docker.errors.NotFound:
            status = 'not_found'
        finally:
            self.finish(json.dumps({
                'data': status[0]
            }))




import sys
from glob import glob

import tornado.ioloop
from tornado.web import Application
from jupyter_client import manager as kernel_manager, kernelspec
from jupyter_core.paths import jupyter_runtime_dir

from .handlers import MainHandler, NotebookRunnerHandler
from .util import get_config


def start_helper():
    print("FAIR-Cells helper starting.\n", flush=True)

    try:
        config = get_config()

        cell_num = config['index'] + 1
        path = config['path']

        print('Available routes:')
        print('  - /')
        print(f'      This will run cell {cell_num} of {path}.')
        print()
        print('      Available variables:')

        for var in config['variables']:
            var_type = config['variables'][var]
            print(f'        {var} - {var_type}')

        print()
        for notebook in glob('*.ipynb'):
            print(f'  - /notebook/{notebook}?cellIdx=<index>')
            print(f'      This will run cell <index> of {notebook}.')
            print()
            print('      No available variables.')

        print(flush=True)

    except FileNotFoundError:
        print('Unable to load config file. Exiting.', file=sys.stderr)
        return

    app = Application([
        (r"/notebook/(.+\.ipynb)", NotebookRunnerHandler),
        (r"/", MainHandler)
    ], autoreload=True)
    app.listen(8888)

    tornado.ioloop.IOLoop.current().start()



import base64
import json
import os
import importlib

from typing import Optional

from tornado.web import RequestHandler, HTTPError

from .notebook_runner import NotebookRunner
from .util import get_config

class NotebookRunnerHandler(RequestHandler):
    def get(self, path):
        try:
            nr = NotebookRunner(path)
        except FileNotFoundError:
            raise HTTPError(404, path + " not found.")

        cell = self._int_argument('cell')
        cell_results = nr.run_until(cell)

        nr.shutdown()

        if cell is not None and cell >= 0 and cell < len(cell_results):
            self._write_cell(cell_results[cell])
        else:
            self.write({
                'data': cell_results
            })

    def on_finish(self):
        print(end='', flush=True)

    def _write_cell(self, cell, prefer=None):
        self.set_header('Access-Control-Allow-Origin', '*')
        
        if 'display' in cell or 'result' in cell:
            if 'display' in cell:
                data = cell['display']['data']
            else:
                data = cell['result']['data']

            if 'image/png' in data:
                img = data['image/png']

                self.set_header('Content-Type', 'image/png')
                self.write(base64.b64decode(img))
            if 'application/json' in data:
                self.set_header('Content-Type', 'application/json')
                self.write(data['application/json'])
            elif 'text/html' in data:
                #  TODO Render in html-page
                self.set_header('Content-Type', 'text/html')
                self.write('<html><body>' + data['text/html'] + '</body></html>')
            elif 'text/markdown' in data:
                #  TODO Render markdown (maybe)
                self.set_header('Content-Type', 'text/markdown')
                self.write(data['text/markdown'])
            elif 'text/plain' in data:
                self.set_header('Content-Type', 'text/plain')
                self.write(data['text/plain'])
            else:
                self.write('ERROR, found no renderable text.')

        elif 'stderr' in cell:
            self.write(cell['stderr'])

        elif 'stdout' in cell:
            self.write(cell['stdout'])

        elif 'error' in cell:
            self.write(cell['error'])

        else:
            self.set_status(201)

    def _int_argument(self, name, default=None) -> Optional[int]:
        try:
            return int(self.get_argument(name, default))
        except (ValueError, TypeError):
            return default


class MainHandler(NotebookRunnerHandler):
    def _get_inspector(self, kernel, config):
        try:
            inspector_module = importlib.import_module(
                f'.inspection.{kernel}', package="fair-cells")

            return inspector_module.inspector()
        except ModuleNotFoundError:
            return None

    def post(self):
        return self.get()

    def get(self):
        config = get_config()

        idx = config['index']

        try:
            nr = NotebookRunner(config['path'])
        except FileNotFoundError:
            raise HTTPError(404, config['path'] + " not found.")

        runner = self._get_inspector(nr.kernel_name, config)

        if runner:
            nr.set_run_before(idx, runner.run_pre_cell(config, self))

        cell_results = nr.run_until(idx)

        nr.shutdown()

        self._write_cell(cell_results[idx])



def _jupyter_server_extension_paths():
    return [{
        "name": "FAIR-Cells",
        "module": "fair-cells",
        "module_name": "fair-cells"
    }]


def _jupyter_nbextension_paths():
    return [dict(
        section="notebook",
        src="frontend",
        dest="fair-cells",
        require="fair-cells/index")]


def load_jupyter_server_extension(nbapp):
    from notebook.utils import url_path_join

    from .backend.handlers.environment_handler import EnvironmentHandler
    from .backend.handlers.template_handler import TemplateHandler
    from .backend.handlers.build_handler import BuildHandler
    from .backend.handlers.build_docker_file_handler import BuildDockerFileHandler
    from .backend.handlers.docker_publish_handler import DockerPublishHandler
    from .backend.handlers.docker_images_handler import DockerImagesHandler
    from .backend.handlers.login_docker_repository_handler import LoginDockerRepositoryHandler
    from .backend.handlers.command_handler import CommandHandler
    from .backend.handlers.inspect_handler import InspectHandler

    nbapp.log.info("FAIR-Cells loaded.")

    web_app = nbapp.web_app
    base = web_app.settings['base_url']

    host_pattern = '.*$'
    build_pattern = url_path_join(base, '/dj/notebook/(.*)/build')
    build_docker_file_pattern = url_path_join(base, '/dj/notebook/(.*)/build_docker_file')
    images_pattern = url_path_join(base, '/dj/notebook/(.*)/images')
    login_docker_repository_pattern = url_path_join(base, '/dj/notebook/(.*)/login')
    publish_docker_image_pattern = url_path_join(base, '/dj/notebook/(.*)/publish')
    image_command_pattern = url_path_join(base, '/dj/image/(.*)/command/(.*)')
    environment_pattern = url_path_join(base, '/dj/notebook/(.*)/environment')
    inspect_pattern = url_path_join(base, '/dj/notebook/(.*)/inspect/(.*)')

    template_pattern = url_path_join(base, r'/dj/templates/(.*\.(?:html|js|css))')

    web_app.add_handlers(host_pattern, [
        (build_pattern, BuildHandler),
        (build_docker_file_pattern, BuildDockerFileHandler),
        (images_pattern, DockerImagesHandler),
        (login_docker_repository_pattern, LoginDockerRepositoryHandler),
        (publish_docker_image_pattern, DockerPublishHandler),
        (image_command_pattern, CommandHandler),
        (environment_pattern, EnvironmentHandler),
        (inspect_pattern, InspectHandler),
        (template_pattern, TemplateHandler)])





import json
import logging
import requests
import sys

from .base_handler import BaseHandler
from ..container.docker_service import DockerService

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)



class DockerImagesHandler(BaseHandler):

    def post(self, path):
        body = self.get_json_body()

        ds = DockerService()
        images = ds.get_local_images(tag='cloudcells')
        logging.info("resp: " + str(images))
        self.finish(json.dumps(images))





"""
POC: Create a Docker container from a Python script.
"""
import sys
import os
import random
import string

import docker

import traceback

import logging
logger = logging.getLogger('DockerService')
logger.setLevel(logging.DEBUG)


#  Small hack so we can use a dockerfile string instead of having to write it
#  to disk first. See:
#  https://github.com/docker/docker-py/issues/2105#issuecomment-613685891
docker.api.build.process_dockerfile = \
    lambda dockerfile, _: ('Dockerfile', dockerfile)



class DockerService:
    def __init__(self,):
        # self.folder = folder
        # self.name = image_name
        # self.base_image = base_image
        self.client = docker.from_env()

    def get_dockerfile(self,base_image) -> str:
        return (
            "\n".join([
                f'FROM {base_image}',
                f'USER root',
                f'WORKDIR /src/',

                f'COPY ./fair-cells/ /src/fair-cells/',
                f"RUN pip install -r /src/fair-cells/helper/helper_requirements.txt",

                f'COPY ./environment.yml /src/environment.yml',
                f"RUN conda env update --file environment.yml --name base",

                f"USER $NB_UID",

                f'COPY ./notebook.ipynb /src/notebook.ipynb',
                f'COPY ./nb_helper_config.json /src/nb_helper_config.json',

                f'ENTRYPOINT ["python", "-m", "fair-cells"]'
            ])
        )

    def build_container(self, dockerfile: str, folder, image_name) -> docker.api.image.ImageApiMixin:
        #  Save working directory so we can reset it later
        wd = os.getcwd()
        os.chdir(folder)
        logging.info("getcwd: " + str(wd))
        try:
            logging.info("dockerfile: "+str(dockerfile))
            logging.info("Start building container. self.client.images.build")

            image, log = self.client.images.build(tag='cloudcells/'+image_name,
                                            path='.',
                                            dockerfile=dockerfile,
                                            rm=True,
                                            nocache=True)
        except Exception as e:
            logging.error(traceback.format_exc())
        finally:
            #  Change back
            os.chdir(wd)
        logging.info("Returning image and log: "+str(log))
        return image, log


    def run_container(self, port=10000, image=None) -> docker.api.container.ContainerApiMixin:
        try:
            self.stop_image(image=image)
        except docker.errors.NotFound:
            pass
        logging.info("containers.run name: " + image + " ports: " + str(port))
        container_out = self.client.containers.run(image,
                                   # name=image,
                                   ports={'8888': port},
                                   remove=True,
                                   detach=True)
        return container_out

    def login(self,url=None,username=None,token=None):
        logging.info('username: '+username +'password: '+token + 'registry:'+url)
        resp = self.client.login(username=username, password=token, registry=url)
        logging.info("resp: " + str(resp))
        return resp

    def get_local_images(self,tag=None):
        images = []
        for image in self.client.images.list():
            if image.tags:
                if tag:
                    for im_tag in image.tags:
                        if tag in im_tag:
                            ret_image = {'name': im_tag}
                            # ret_image['short_id'] = image.short_id
                            images.append(ret_image)
                elif not tag:
                    ret_image = {'name': image.tags[0]}
                    # ret_image['short_id'] = image.short_id
                    images.append(ret_image)
        return images

    def push(self,images):
        logging.info("images: " + str(images))
        results = []
        for image in images:
            logging.info("Pushing: " + str(image.split(':')[0]))
            re = self.client.images.push(image.split(':')[0])
            logging.info("re: " + str(re))
            results.append(re)
        return results

    def get_image_status(self, image=None):
        running_containers = self.get_container(image_name=image)
        stats = []
        if running_containers:
            for cont in running_containers:
                stats.append( cont.status )
        return stats

    def stop_image(self, image=None):
        running_containers = self.get_container(image_name=image)
        stats = []
        if running_containers:
            for cont in running_containers:
                stats.append(cont.stop(timeout=1))
        return stats

    def get_container(self, image_name=None):
        running_containers = self.client.containers.list()
        if not image_name:
            return running_containers
        if running_containers:
            for cont in running_containers:
                    for tag in cont.image.tags:
                        if tag.split(':')[0] == image_name:
                            return [cont]


import json
import os
import tempfile
import shutil
import importlib

from typing import Optional

import docker
from notebook.base.handlers import IPythonHandler, APIHandler, HTTPError
from notebook.utils import url_path_join
from pigar.core import parse_packages

from ..container.docker_service import DockerService
from .base_handler import BaseHandler
from .environment_handler import BASE_STRING


import logging

logger = logging.getLogger('BuildHandler')
logger.setLevel(logging.DEBUG)

# create console handler and set level to debug
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# add formatter to ch
ch.setFormatter(formatter)

# add ch to logger
logger.addHandler(ch)


def create_config(notebook_path, cell_index, variables):
    return json.dumps({
        'path': notebook_path,
        'index': cell_index,
        'variables': variables
    })


class BuildHandler(BaseHandler):
    def post(self, path):        
        notebook = self.contents_manager.get(path, content=True)
        notebook_path = os.path.join(os.getcwd(), path)

        notebook_name = "notebook.ipynb"

        body = self.get_json_body()

        image_name = body.get('imageName')
        base_image = body.get('baseImage')
        cell_index = int(body.get('cellIndex'))
        variables = body.get('variables', {})
        logging.info("image_name: " + str(image_name))
        logging.info("base_image: " + str(base_image))
        logging.info("cell_index: " + str(cell_index))
        logging.info("variables: " + str(variables))

        if image_name is None or base_image is None or cell_index is None:
            raise HTTPError(400, 'abc')

        requirements = body.get('environment', BASE_STRING)

        #  Create a temporary dir which will be our build context.
        with tempfile.TemporaryDirectory() as tmpdir:
            shutil.copyfile(notebook_path, tmpdir + "/" + notebook_name)

            #  Find the location of the FAIR-Cells module on disk
            #  So it can copy & install it in the container.
            dirname = os.path.dirname(__file__)
            nested_levels = len(__name__.split('.')) - 2
            module_path = os.path.join(dirname + '/..' * nested_levels)

            #  Copy helper to build context.
            shutil.copytree(module_path, 
                tmpdir + "/fair-cells/",
                ignore=shutil.ignore_patterns('.ipynb_checkpoints', '__pycache__'))

            with open(tmpdir + "/environment.yml", "a") as reqs:
                reqs.write(requirements)

            with open(tmpdir + "/nb_helper_config.json", "a") as cfg:
                config = create_config(notebook_name, cell_index, variables)
                cfg.write(config)

            with open(tmpdir + "/.dockerignore", "a") as ignore:
                ignore.write("**/backend\n")
                ignore.write("**/frontend\n")

            logging.info("image_name: " + str(image_name))
            cc = DockerService()

            try:
                logging.info("Start building container")
                _, log = cc.build_container(cc.get_dockerfile(base_image),tmpdir,image_name)
                logging.info("Finish building container")
            except docker.errors.BuildError as be:
                logger.error(str(be))
                logger.error(str(be.build_log))
                log = be.build_log

        logs = "".join([l['stream'] if 'stream' in l else '' for l in log])
    

        self.finish(json.dumps({
            'logs': logs
        }))



from typing import List

class InspectorBase:
    def available(self):
        return False

    def get_variables(self, notebook, cell_index) -> List[str]:
        return []

    def run_pre_cell(self, config, req) -> str:
        """Returns code as string that will run before the final cell does."""
        return ''



import json
import os
import importlib

from typing import Optional

import docker
from notebook.base.handlers import APIHandler, HTTPError

from ..container.docker_service import DockerService
from .base_handler import BaseHandler

class InspectHandler(BaseHandler):
    def _get_inspector(self, notebook):
        kernel = notebook['content']['metadata']['kernelspec']['name']

        try:
            inspector_module = importlib.import_module(
                f'.inspection.{kernel}', package="fair-cells")

            inspector = inspector_module.inspector()
            
            if inspector.available():
                return inspector
        except ModuleNotFoundError:
            pass

        raise HTTPError(501, 'inspector_unavailable')


    def get(self, path, command):
        notebook = self.contents_manager.get(path, content=True)
        inspector = self._get_inspector(notebook)

        if command == 'available':
            self.set_status(201)
            self.finish()
        elif command == 'variables':
            cell_index = self._int_argument('cellIdx')

            if cell_index is None:
                raise HTTPError(400, 'invalid_cell_idx')

            variables = inspector.get_variables(notebook, cell_index)

            self.finish(json.dumps({
                'data': variables
            }))
        elif command == "inspector.html":
            cell_index = self._int_argument('cellIdx')

            if cell_index is None:
                raise HTTPError(400, 'invalid_cell_idx')

            self.render("inspector.html", **{
                'variables': inspector.get_variables(notebook, cell_index)
            })
        else:
            raise HTTPError(400, 'no_cmd')

    def post(self, path, command):
        pass

    def get_template_path(self):
        dirname = os.path.dirname(__file__)

        return os.path.join(dirname + "/../templates")



import setuptools
import os
from shutil import copyfile


short_desc = "FAIR-Cells allows the user to interactively create a Docker image from a Jupyter Notebook."

try:
    with open("./README.md", "r") as rm:
        long_description = rm.read()
except FileNotFoundError:
    long_description = short_desc


setuptools.setup(
    name="FAIR-Cells",
    version="1.0.12",
    author="Wilco Kruijer, Zhiming Zhao",
    author_email="Z.Zhao@uva.nl",
    description=short_desc,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/QCDIS/FAIRCells",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        "docker>=4.2.0",
        "pigar>=0.10.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from tornado.web import RequestHandler, HTTPError
from tornado import template

import os

class TemplateHandler(RequestHandler):
    def get(self, path):
        args = None

        if path == 'form.html':
            args = self._form_html()
        elif path == 'inspector.html':
            args = self._inspector_html()
        try:
            if args is None:
                args = {}

            self.render(path, **args)
        except FileNotFoundError as e:
            raise HTTPError(404)


    def _inspector_html(self):
        return {
            'variables': []
        }
        

    def _form_html(self):
        base_images = [
            ('Base Notebook', 'jupyter/base-notebook'),
            ('Minimal Notebook', 'jupyter/minimal-notebook'),
            ('R Notebook', 'jupyter/r-notebook'),
            ('Scipy Notebook', 'jupyter/scipy-notebook'),
            ('Data Science Notebook', 'jupyter/datascience-notebook')
        ]

        base_images.reverse()

        return {
            'images': base_images
        }

    
    def get_template_path(self):
        dirname = os.path.dirname(__file__)

        return os.path.join(dirname + "/../templates")


import json
import logging

from .base_handler import BaseHandler
from ..container.docker_service import DockerService

logger = logging.getLogger('DockerPublishHandler')
logger.setLevel(logging.DEBUG)



class DockerPublishHandler(BaseHandler):

    def post(self, path):
        body = self.get_json_body()
        images = body.get('images')
        print("images: " + str(images))
        logging.info("images: " + str(images))
        ds = DockerService()
        resp = ds.push(images)

        self.finish(json.dumps(resp))




from typing import Optional

from notebook.base.handlers import IPythonHandler, APIHandler, HTTPError

class BaseHandler(APIHandler):
    def check_xsrf_cookie(self):
        # Hack to not get missing _xsrf error. Probably makes the routes insecure.

        # [W 14:42:46.388 NotebookApp] 403 POST /notebook/empty.ipynb/build (127.0.0.1): '_xsrf' argument missing from POST
        # [W 14:42:46.389 NotebookApp] '_xsrf' argument missing from POST
        return

    def _int_body(self, name, default: int =None) -> Optional[int]:
        try:
            return int(self.get_json_body()[name])
        except (ValueError, TypeError, KeyError):
            return default

    def _int_argument(self, name, default: int = None) -> Optional[int]:
        try:
            return int(self.get_argument(name, default))
        except (ValueError, TypeError):
            return default
